;  ------------------------------------------------------------------

;  These procedures open and maintain an "event trace" window,
;  displaying the information received by the WAIT procedure.

;  To use the WAIT event tracing:

;  1)  Call Wait_Trace_Init() in your program initialization,
;      prior to your WAIT command.

;  2)  Call Wait_Trace(triggertype, eventrecord, cyclenum)
;      as the first statement in your WAIT procedure.  Pass into
;      it the same three parameters that the WAIT procedure receives.

;  3)  A dynamic array, EVTRACE_PARAMS, is created by Wait_Trace_Init()
;      and used by Wait_Trace().  Therefore, you must make sure this
;      array is globabl enough to be visible by both procedures.

;  ------------------------------------------------------------------

Proc Wait_Trace_Init()

;  Intializes WAIT event tracing, by opening a canvas window to display
;  the trace information.

Private
   ch,          ;canvas height
   h,           ;height of the event trace window
   sb,          ;SYSINFO dynamic array
   sh,          ;screen height
   wb,          ;dynamic array with attributes for event trace window
   wh           ;window handle for the event trace window

;  Determine the screen dimensions

SysInfo to sb                   ;get system information
sh = sb["ScreenHeight"]         ;screen height

;  Determine the attributes of the event trace window

DynArray wb[]                           ;dynamic array for attributes
wb["Title"]        = "Event Trace"      ;fill in the attributes
ch = 200
wb["CanvasHeight"] = ch
wb["CanvasWidth"]  = 76

If sh<=25 then
   h = int(sh/3)
Else
   h = sh-27
Endif

;  Create the event trace window

Window Create Floating @ sh-h-2,1 Height h Width 78
              Attributes wb To wh

;  Remember information about the event tracing

DynArray evtrace_params[]             ;dynamic array for parameters
evtrace_params["Window Handle"] = wh  ;store parameters
evtrace_params["Window Title "+strval(wh)] = wb["Title"]
evtrace_params["Next Row"] = 0
evtrace_params["Canvas Height"] = ch
evtrace_params["Window Height"] = h
evtrace_params["Alt Color"] = False

EndProc

;  ------------------------------------------------------------------

Proc Wait_Trace(trigger_type, event_record, cycle_number)

;  Displays the information about a single WAIT event in the
;  event-trace window created by WAIT_TRACE_INIT().

;  The parameters trigger_type, event_record, and cycle_number,
;  are the same three parameters passed by the WAIT command into
;  the WAIT procedure.

Private
   et,          ;event type
   em,          ;event message
   ew,          ;window handle being acted upon
   t,           ;the line of text to be displayed
   tw,          ;window handle of the trace window
   wb,          ;dynamic array of window attributes
   wt           ;title of the event window

;  Ignore mouse actions on the event trace window

tw = evtrace_params["Window Handle"]    ;event trace window handle
If getwindow()=tw and                   ;we're already on the trace window
   trigger_type="EVENT" and             ;it's an event
   event_record["Type"]="MOUSE" and     ;it's a mouse event
   windowat(event_record["Row"],event_record["Col"])=tw ;at the same window
      then Return                       ;ignore it
Endif

;  Build the line of text describing the event, 
;  to be displayed in the window.

ew = -1                                 ;no event window identified yet
t = strval(cycle_number)                ;start out with the cycle #
t = t + " " + trigger_type              ;add the trigger type

;  For table-oriented events, add the table name

If trigger_type<>"EVENT" and trigger_type<>"ARRIVEWINDOW" then
   t = t + " " + table()                    ;add the table name
Endif

;  Add information about the event

et = event_record["Type"]               ;the event type
t = t + " " + event_record["Type"]      ;add the event type

Switch
   Case et="MOUSE":                             ;a mouse event
      t = t + " " + event_record["Action"]      ;add mouse event info
            + " " + event_record["Buttons"]
            + iif(event_record["DoubleClick"]," DOUBLECLICK"," ")
      ew = windowat(event_record["Row"],event_record["Col"]) ;mouse window
   Case et="KEY":                                     ;a key event
      t = t + " KeyCode=" + strval(event_record["KeyCode"])  ;add key info
            + " ScanCode=" + strval(event_record["ScanCode"])
   Case et="MESSAGE":
      em = event_record["Message"]              ;the type of message
      t = t + " " + em                          ;add the message type
      If em="MENUSELECT" or em="MENUKEY" then   ;a menu event
         t = t + " " + event_record["MenuTag"]  ;add the selected menu tag
         If em="MENUKEY" then                   ;a menu key event
            t = t + " KeyCode=" + strval(event_record["KeyCode"])
                  + " ScanCode=" + strval(event_record["ScanCode"])

         Endif
      Else                                      ;a non-menu message event
         ew = GetWindow()                       ;the window it applies to
      Endif
EndSwitch

;  Determine the title of the current window, if appropriate

If trigger_type="ARRIVEWINDOW" then
   ew = GetWindow()                     ;get the new window handle
Endif

If ew>=0 then                           ;there is a window
   If iswindow(ew) then                 ;the window handle exists
      If isassigned(evtrace_params["Window Title "+strval(ew)]) then
         wt = evtrace_params["Window Title "+strval(ew)]
      Else                              ;we don't know its name yet
         Window GetAttributes ew to wb
         wt = wb["Title"]               ;get the window title
         evtrace_params["Window Title "+strval(ew)] = wt ;save the title
      Endif
      wt = "Window=\"" + wt + "\""      ;build the string
   Else
      wt = "No Window Here"
   Endif
   t = t + " " + wt                     ;add the window title
Endif

t = t + " ec=" + strval(errorcode())
If errorcode()<>0 then
   t = t + " " + errormessage()
Endif

;  Display the text

Event_Display(t)

EndProc

;  ------------------------------------------------------------------

Proc Event_Display(t)

;  Display the text contained in "t"

Private
   orig_canvas, ;window handle of the current canvas window
   r,           ;the canvas row on which we're writing
   tw           ;window handle of the trace window

;  Switch to the event canvas

tw = evtrace_params["Window Handle"]    ;event trace window handle
orig_canvas = GetCanvas()               ;remember original canvas #
If tw<>orig_canvas then                 ;some other canvas is current
   SetCanvas tw                         ;make event trace the current canvas
Endif

r = evtrace_params["Next Row"]                 ;row # to write on
If r+5 > evtrace_params["Canvas Height"] then  ;we're at bottom of canvas
   Clear                                       ;clear the canvas
   r = 0                                       ;reset row counter
Endif

If not evtrace_params["Alt Color"] then  ;write line in primary color
   Style Attribute 10                    ;primary color:  green on black
Else                                     ;write line in secondary color
   Style Attribute 112                   ;secondary color:  black on gray
Endif

@ r,0 ?? t                               ;write the text string

r = row()

;  WINDOW SCROLL fails if the current window is a memo field
;Window Scroll tw to max(r-evtrace_params["Window Height"]+4,0),0
Cursor Normal Cursor Off   ;causes scrolling in all circumstances

evtrace_params["Next Row"] = r + 1       ;where to write the next row
evtrace_params["Alt Color"] = not evtrace_params["Alt Color"] ;reverse color

If orig_canvas<>tw then          ;need to restore the original canvas
   If orig_canvas>1 then         ;not the full-screen canvas
      SetCanvas orig_canvas      ;make event trace the current canvas
   Else                          ;the full-screen canvas
      SetCanvas Default          ;make the full-screen canvas current
   Endif
Endif

EndProc

;  ------------------------------------------------------------------

